"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.MessageSchemaContainer = void 0;
class MessageSchemaContainer {
    messageDefinitions;
    messageSchemas;
    messageTypeField;
    constructor(options) {
        this.messageTypeField = options.messageTypeField;
        this.messageSchemas = this.resolveSchemaMap(options.messageSchemas);
        this.messageDefinitions = this.resolveDefinitionsMap(options.messageDefinitions ?? []);
    }
    resolveSchema(
    // biome-ignore lint/suspicious/noExplicitAny: This is expected
    message) {
        const schema = this.messageSchemas[message[this.messageTypeField]];
        if (!schema) {
            return {
                error: new Error(`Unsupported message type: ${message[this.messageTypeField]}`),
            };
        }
        return {
            result: schema,
        };
    }
    resolveSchemaMap(supportedSchemas) {
        return supportedSchemas.reduce((acc, schema) => {
            // @ts-ignore
            acc[schema.shape[this.messageTypeField].value] = schema;
            return acc;
        }, {});
    }
    resolveDefinitionsMap(supportedDefinitions) {
        return supportedDefinitions.reduce((acc, definition) => {
            // @ts-ignore
            acc[definition.publisherSchema.shape[this.messageTypeField].value] = definition;
            return acc;
        }, {});
    }
}
exports.MessageSchemaContainer = MessageSchemaContainer;
//# sourceMappingURL=MessageSchemaContainer.js.map